@@ -95,6 +95,7 @@ const AppMenuButton = new Lang.Class({
         this._startingApps = [];
 
         this._menuManager = panel.menuManager;
+        this._gtkSettings = Gtk.Settings.get_default();
         this._targetApp = null;
         this._appMenuNotifyId = 0;
         this._actionGroupNotifyId = 0;
@@ -123,11 +124,14 @@ const AppMenuButton = new Lang.Class({
         this._arrow = PopupMenu.arrowIcon(St.Side.BOTTOM);
         this._container.add_actor(this._arrow);
 
-        this._visible = !Main.overview.visible;
+        this._visible = this._gtkSettings.gtk_shell_shows_app_menu &&
+                        !Main.overview.visible;
         if (!this._visible)
             this.actor.hide();
         this._overviewHidingId = Main.overview.connect('hiding', Lang.bind(this, this._sync));
         this._overviewShowingId = Main.overview.connect('showing', Lang.bind(this, this._sync));
+        this._showsAppMenuId = this._gtkSettings.connect('notify::gtk-shell-shows-app-menu',
+                                                         Lang.bind(this, this._sync));
 
         this._stop = true;
 
@@ -305,7 +309,9 @@ const AppMenuButton = new Lang.Class({
             }
         }
 
-        let visible = (this._targetApp != null && !Main.overview.visibleTarget);
+        let visible = (this._targetApp != null &&
+                       this._gtkSettings.gtk_shell_shows_app_menu &&
+                       !Main.overview.visibleTarget);
         if (visible)
             this.show();
         else
@@ -378,6 +384,10 @@ const AppMenuButton = new Lang.Class({
             Main.overview.disconnect(this._overviewShowingId);
             this._overviewShowingId = 0;
         }
+        if (this._showsAppMenuId > 0) {
+            this._gtkSettings.disconnect(this._showsAppMenuId);
+            this._showsAppMenuId = 0;
+        }
         if (this._switchWorkspaceNotifyId > 0) {
             global.window_manager.disconnect(this._switchWorkspaceNotifyId);
             this._switchWorkspaceNotifyId = 0;
@@ -642,14 +652,50 @@ const PanelCorner = new Lang.Class({
     }
 });
 
+const AggregateLayout = new Lang.Class({
+    Name: 'AggregateLayout',
+    Extends: Clutter.BoxLayout,
+
+    _init: function(params) {
+        if (!params)
+            params = {};
+        params['orientation'] = Clutter.Orientation.VERTICAL;
+        this.parent(params);
+
+        this._sizeChildren = [];
+    },
+
+    addSizeChild: function(actor) {
+        this._sizeChildren.push(actor);
+        this.layout_changed();
+    },
+
+    vfunc_get_preferred_width: function(container, forHeight) {
+        let themeNode = container.get_theme_node();
+        let minWidth = themeNode.get_min_width();
+        let natWidth = minWidth;
+
+        for (let i = 0; i < this._sizeChildren.length; i++) {
+            let child = this._sizeChildren[i];
+            let [childMin, childNat] = child.get_preferred_width(forHeight);
+            minWidth = Math.max(minWidth, childMin);
+            natWidth = Math.max(minWidth, childNat);
+        }
+        return [minWidth, natWidth];
+    }
+});
+
 const AggregateMenu = new Lang.Class({
     Name: 'AggregateMenu',
     Extends: PanelMenu.Button,
 
     _init: function() {
-        this.parent(0.0, _("Settings"), false);
+        this.parent(0.0, C_("System menu in the top bar", "System"), false);
         this.menu.actor.add_style_class_name('aggregate-menu');
 
+        let menuLayout = new AggregateLayout();
+        this.menu.box.set_layout_manager(menuLayout);
+
         this._indicators = new St.BoxLayout({ style_class: 'panel-status-indicators-box' });
         this.actor.add_child(this._indicators);
 
@@ -698,6 +744,11 @@ const AggregateMenu = new Lang.Class({
         this.menu.addMenuItem(this._rfkill.menu);
         this.menu.addMenuItem(this._power.menu);
         this.menu.addMenuItem(this._system.menu);
+
+        menuLayout.addSizeChild(this._location.menu.actor);
+        menuLayout.addSizeChild(this._rfkill.menu.actor);
+        menuLayout.addSizeChild(this._power.menu.actor);
+        menuLayout.addSizeChild(this._system.menu.actor);
     },
 });
 
